const { pool } = require("../config/database/database.config");

/*
Repository for login and changelog tables
*/

/*
Add a row to the userLogin table indicating a successful 
login.
*/
async function logSuccessfulLogin(userId, timestamp) {
  try {
    // SQL query to insert the login info
    const query = `
        INSERT INTO userLogin (eventTimestamp, userId) VALUES (?,?)
    `;

    // make query
    const [result] = await pool.query(query, [timestamp, userId]);
    console.log("Successfully logged log in.");

    // return the inserted id
    return result.insertId;
  } catch (error) {
    // throw error to be caught by the service
    throw error;
  }
}

async function getLoginDataTimerange(start, end) {
  try {
    // get all login data within the start and end timerange
    const query = `
      SELECT *
      FROM userLogin
      WHERE eventTimestamp BETWEEN ? AND ?
      ORDER BY eventTimestamp DESC
    `;
    const [rows] = await pool.query(query, [start, end]);
    return rows;
  } catch (error) {
    // throw error to be caught in service
    console.error("Error fetching login data:", error);
    throw error;
  }
}

async function getLoginDataByUser() {}

/*
Insert a row into changeLog which keeps track of every time the database is changed.
*/
async function logDatabaseChange(
  tableName,
  recordId,
  userId,
  changeType,
  changedColumns,
  oldValues,
  newValues
) {
  const query = `
    INSERT INTO changeLog (changedTable, recordId, userId, changeType, changedColumns, oldValues, newValues)
    VALUES (?, ?, ?, ?, ?, ?, ?)
  `;
  const values = [
    tableName,
    recordId,
    userId,
    changeType,
    changedColumns,
    oldValues,
    newValues,
  ];

  try {
    await pool.query(query, values);
    console.log("Change logged successfully.");
  } catch (error) {
    // throw error to be caught by service
    console.log("activity repository error when logging change: ", error);
    throw error;
  }
}

async function getDatabaseChangeTimerange() {}

async function getDatabaseChangeByUser() {}

module.exports = {
  logSuccessfulLogin,
  getLoginDataTimerange,
  logDatabaseChange,
};
